/**
 * Copyright 2004-2012 Crypto-Pro. All rights reserved.
 * Этот файл содержит информацию, являющуюся
 * собственностью компании Крипто-Про.
 *
 * Любая часть этого файла не может быть скопирована,
 * исправлена, переведена на другие языки,
 * локализована или модифицирована любым способом,
 * откомпилирована, передана по сети с или на
 * любую компьютерную систему без предварительного
 * заключения соглашения с компанией Крипто-Про.
 */
package CAdES;

import org.bouncycastle.cms.CMSSignedData;
import org.bouncycastle.cms.SignerInformation;
import org.bouncycastle.cms.SignerInformationStore;
import ru.CryptoPro.CAdES.CAdESSignature;
import ru.CryptoPro.CAdES.CAdESSigner;
import ru.CryptoPro.CAdES.CAdESType;
import ru.CryptoPro.JCP.JCP;
import ru.CryptoPro.JCP.tools.Array;

import java.io.FileInputStream;
import java.security.cert.CertificateFactory;
import java.security.cert.X509CRL;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

/**
 * Пример усовершенствования подписи CAdES-BES до CAdES-X Long Type 1.
 * 
 * @author Yevgeniy, 17/04/2012
 *
 */
public class EnhanceExample {

	/**
	 * @param args
	 */
	public static void main(String[] args) {

		try {
			
			Collection<X509Certificate> chain = new ArrayList<X509Certificate>();
			Configuration.loadConfiguration(chain);

			// 1. Загрузка и "проверка" подписи. 
		
			// Читаем подпись из файла.
			byte[] cadesCms = Array.readFile(Configuration.TEST_DIR + Configuration.SIGNATURE_FILENAME);
				
			// Подпись в тесте была совмещенная, потому данные равны null. Предположим, что
			// подписей несколько, тогда лучше указать тип null и положиться на самоопределение
			// типа подписи.
			CAdESSignature cadesSignature = new CAdESSignature(cadesCms, null, null);

			// Список CRL.
			List<X509CRL> crlList = null;
			
			// Если задан CRL, то читаем его из файла.
			if (Configuration.CRL_FILENAME != null) {
							
				X509CRL crl = (X509CRL) CertificateFactory.getInstance("X.509")
					.generateCRL(new FileInputStream(Configuration.CRL_FILENAME));
					
				crlList = Collections.singletonList(crl);
				cadesSignature.verify(chain, crlList);
							
			} else {
				cadesSignature.verify(chain);
			}
			
			// Список всех подписантов в исходной подписи.
			Collection<SignerInformation> srcSignerInfos = new ArrayList<SignerInformation>();
						
			for (CAdESSigner signer : cadesSignature.getCAdESSignerInfos()) {
				srcSignerInfos.add(signer.getSignerInfo());
			}
			
			// 2. Усовершенствование подписи.
		
			// Получаем только первую CAdES-BES подпись, усовершенствуем ее.
			// Остальных не трогаем.
			CMSSignedData srcSignedData = cadesSignature.getSignedData();
			CAdESSigner srcSigner = cadesSignature.getCAdESSignerInfo(0);
			
			// Исключаем ее из исходного списка, т.к. ее место займет усовершенствованная
			// подпись.
			srcSignerInfos.remove(srcSigner.getSignerInfo());
			
			// Усовершенствуем CAdES-BES до CAdES-X Long Type 1.
			srcSigner.enhance(JCP.PROVIDER_NAME, JCP.GOST_DIGEST_OID, chain, 
				Configuration.TSA_ADDRESS, CAdESType.CAdES_X_Long_Type_1);
						
			// Подписант усовершенствованной подписи.
			SignerInformation enhSigner = srcSigner.getSignerInfo();
			
			// Добавляем его в исходный список подписантов.
			srcSignerInfos.add(enhSigner);
			
			// Список подписантов.
			SignerInformationStore dstSignerInfoStore = new SignerInformationStore(srcSignerInfos);
			
			// Обновляем исходную подпись c ее начальным списком подписантов на тот же,
			// но с первым подписантом с усовершенствованной подписью.
			CMSSignedData dstSignedData = 
				CMSSignedData.replaceSigners(srcSignedData, dstSignerInfoStore);
			
			Array.writeFile(Configuration.TEST_DIR + "enhanced_" + Configuration.SIGNATURE_FILENAME, 
				dstSignedData.getEncoded());
		
			// 3. Проверка усовершенствованной подписи.
		
			// Проверяем подпись.
			cadesSignature = new CAdESSignature(dstSignedData.getEncoded(), null, null);
			cadesSignature.verify(chain, crlList);
			
			Configuration.printSignatureInfo(cadesSignature);
		
		} catch (Exception e) {
            Configuration.printCAdESException(e);
        }
	}
}
